@@ -28,14 +28,14 @@ from turbinia import config
 from turbinia import TurbiniaException
 
 config.LoadConfig()
-if config.GCS_OUTPUT_PATH and config.GCS_OUTPUT_PATH.lower() is not 'none':
+if config.GCS_OUTPUT_PATH and config.GCS_OUTPUT_PATH.lower() != 'none':
   from google.cloud import storage
   from google.cloud import exceptions
 
 log = logging.getLogger('turbinia')
 
 
-class OutputManager(object):
+class OutputManager:
   """Manages output data.
 
   Manages the configured output writers.  Also saves and retrieves evidence data
@@ -126,7 +126,7 @@ class OutputManager(object):
         log.info(
             'Retrieving copyable evidence data from {0:s}'.format(
                 evidence_.saved_path))
-        evidence_.local_path = writer.copy_from(evidence_.saved_path)
+        evidence_.source_path = writer.copy_from(evidence_.saved_path)
     return evidence_
 
   def save_evidence(self, evidence_, result=None):
@@ -143,7 +143,7 @@ class OutputManager(object):
       TurbiniaException: If serialization or writing of evidence config fails
     """
     path, path_type, local_path = self.save_local_file(
-        evidence_.local_path, result)
+        evidence_.source_path, result)
 
     if evidence_.save_metadata:
       metadata = evidence_.config.copy()
@@ -153,7 +153,7 @@ class OutputManager(object):
         json_str = json.dumps(metadata)
       except TypeError as exception:
         raise TurbiniaException(
-            'Could not serialize Evidence config for {0:s}: {1:s}'.format(
+            'Could not serialize Evidence config for {0:s}: {1!s}'.format(
                 evidence_.name, exception))
 
       try:
@@ -162,7 +162,7 @@ class OutputManager(object):
           file_handle.write(json_str.encode('utf-8'))
       except IOError as exception:
         raise TurbiniaException(
-            'Could not write metadata file {0:s}: {1:s}'.format(
+            'Could not write metadata file {0:s}: {1!s}'.format(
                 metadata_path, exception))
 
       self.save_local_file(metadata_path, result)
@@ -171,11 +171,12 @@ class OutputManager(object):
     # where tasks are saving evidence into the temp dir, we'll get the newly
     # copied version from the saved output path.
     if local_path:
-      evidence_.local_path = local_path
+      evidence_.source_path = local_path
     evidence_.saved_path = path
     evidence_.saved_path_type = path_type
-    log.info(
-        'Saved copyable evidence data to {0!s}'.format(evidence_.saved_path))
+    if evidence_.saved_path:
+      log.info(
+          'Saved copyable evidence data to {0:s}'.format(evidence_.saved_path))
     return evidence_
 
   def save_local_file(self, file_, result):
@@ -220,7 +221,7 @@ class OutputManager(object):
     self.is_setup = True
 
 
-class OutputWriter(object):
+class OutputWriter:
   """Base class.
 
   By default this will write the files the Evidence objects point to along with
@@ -361,18 +362,20 @@ class LocalOutputWriter(OutputWriter):
     if self.local_output_dir in os.path.commonprefix([file_path,
                                                       destination_file]):
       log.debug(
-          'Not copying file {0:s} in output dir {1:s}'.format(
+          'Not copying source file {0:s} already in output dir {1:s}'.format(
               file_path, self.local_output_dir))
       return None
     if not os.path.exists(file_path):
-      log.warning('File [{0:s}] does not exist.'.format(file_path))
+      log.warning('Source file [{0:s}] does not exist.'.format(file_path))
       return None
     if os.path.exists(destination_file):
       log.warning(
-          'New file path [{0:s}] already exists.'.format(destination_file))
+          'Target output file path [{0:s}] already exists.'.format(
+              destination_file))
       return None
 
     shutil.copy(file_path, destination_file)
+    log.debug('Copied file {0:s} to {1:s}'.format(file_path, destination_file))
     return destination_file
 
   def copy_to(self, source_file):
