@@ -27,8 +27,8 @@ import sys
 import uuid
 
 from turbinia import config
+from turbinia import TurbiniaException
 from turbinia.config import logger
-from turbinia.lib import libcloudforensics
 from turbinia import __version__
 from turbinia.processors import archive
 from turbinia.output_manager import OutputManager
@@ -107,17 +107,19 @@ def main():
       'text based evidence files with (in extended grep regex format). '
       'This filtered output will be in addition to the complete output')
   parser.add_argument(
-      '-j', '--jobs_whitelist', default=[], type=csv_list,
-      help='A whitelist for Jobs that will be allowed to run (in CSV format, '
+      '-Y', '--yara_rules_file', help='A file containing Yara rules.')
+  parser.add_argument(
+      '-j', '--jobs_allowlist', default=[], type=csv_list,
+      help='An allowlist for Jobs that will be allowed to run (in CSV format, '
       'no spaces). This will not force them to run if they are not configured '
       'to. This is applied both at server start time and when the client makes '
       'a processing request. When applied at server start time the change is '
       'persistent while the server is running.  When applied by the client, it '
       'will only affect that processing request.')
   parser.add_argument(
-      '-J', '--jobs_blacklist', default=[], type=csv_list,
-      help='A blacklist for Jobs we will not allow to run.  See '
-      '--jobs_whitelist help for details on format and when it is applied.')
+      '-J', '--jobs_denylist', default=[], type=csv_list,
+      help='A denylist for Jobs we will not allow to run.  See '
+      '--jobs_allowlist help for details on format and when it is applied.')
   parser.add_argument(
       '-p', '--poll_interval', default=60, type=int,
       help='Number of seconds to wait between polling for task state info')
@@ -125,6 +127,9 @@ def main():
       '-t', '--task',
       help='The name of a single Task to run locally (must be used with '
       '--run_local.')
+  parser.add_argument(
+      '-T', '--debug_tasks', action='store_true',
+      help='Show debug output for all supported tasks', default=False)
   parser.add_argument(
       '-w', '--wait', action='store_true',
       help='Wait to exit until all tasks for the given request have completed')
@@ -137,6 +142,10 @@ def main():
   parser_config.add_argument(
       '-f', '--file_only', action='store_true', help='Print out file path only')
 
+  #Sends Test Notification
+  parser_testnotify = subparsers.add_parser(
+      'testnotify', help='Sends test notification')
+
   # TODO(aarontp): Find better way to specify these that allows for multiple
   # pieces of evidence to be submitted. Maybe automagically create different
   # commands based on introspection of evidence objects?
@@ -144,7 +153,7 @@ def main():
   parser_rawdisk = subparsers.add_parser(
       'rawdisk', help='Process RawDisk as Evidence')
   parser_rawdisk.add_argument(
-      '-l', '--local_path', help='Local path to the evidence', required=True)
+      '-l', '--source_path', help='Local path to the evidence', required=True)
   parser_rawdisk.add_argument(
       '-P', '--mount_partition', default=1, type=int,
       help='The partition number to use when mounting this disk.  Defaults to '
@@ -160,7 +169,7 @@ def main():
   parser_apfs = subparsers.add_parser(
       'apfs', help='Process APFSEncryptedDisk as Evidence')
   parser_apfs.add_argument(
-      '-l', '--local_path', help='Local path to the encrypted APFS evidence',
+      '-l', '--source_path', help='Local path to the encrypted APFS evidence',
       required=True)
   parser_apfs.add_argument(
       '-r', '--recovery_key', help='Recovery key for the APFS evidence.  '
@@ -179,7 +188,7 @@ def main():
   parser_bitlocker = subparsers.add_parser(
       'bitlocker', help='Process Bitlocker Disk as Evidence')
   parser_bitlocker.add_argument(
-      '-l', '--local_path',
+      '-l', '--source_path',
       help='Local path to the encrypted Bitlocker evidence', required=True)
   parser_bitlocker.add_argument(
       '-r', '--recovery_key', help='Recovery key for the Bitlocker evidence.  '
@@ -210,7 +219,7 @@ def main():
       'with. If this is different from the project that Turbinia is running '
       'in, it will be copied to the Turbinia project.')
   parser_googleclouddisk.add_argument(
-      '-P', '--mount_partition', default=0, type=int,
+      '-P', '--mount_partition', default=1, type=int,
       help='The partition number to use when mounting this disk.  Defaults to '
       'the entire raw disk.  Only affects mounting, and not what gets '
       'processed.')
@@ -243,10 +252,14 @@ def main():
       'with. If this is different from the project that Turbinia is running '
       'in, it will be copied to the Turbinia project.')
   parser_googleclouddiskembedded.add_argument(
-      '-P', '--mount_partition', default=0, type=int,
+      '-P', '--mount_partition', default=1, type=int,
       help='The partition number to use when mounting this disk.  Defaults to '
       'the entire raw disk.  Only affects mounting, and not what gets '
       'processed.')
+  parser_googleclouddiskembedded.add_argument(
+      '--embedded_mount_partition', default=1, type=int,
+      help='The partition number to use when mounting this embedded disk image.'
+      ' Defaults to the first partition')
   parser_googleclouddiskembedded.add_argument(
       '-z', '--zone', help='Geographic zone the disk exists in')
   parser_googleclouddiskembedded.add_argument(
@@ -259,7 +272,7 @@ def main():
   parser_rawmemory = subparsers.add_parser(
       'rawmemory', help='Process RawMemory as Evidence')
   parser_rawmemory.add_argument(
-      '-l', '--local_path', help='Local path to the evidence', required=True)
+      '-l', '--source_path', help='Local path to the evidence', required=True)
   parser_rawmemory.add_argument(
       '-P', '--profile', help='Profile to use with Volatility', required=True)
   parser_rawmemory.add_argument(
@@ -272,7 +285,7 @@ def main():
   parser_directory = subparsers.add_parser(
       'directory', help='Process a directory as Evidence')
   parser_directory.add_argument(
-      '-l', '--local_path', help='Local path to the evidence', required=True)
+      '-l', '--source_path', help='Local path to the evidence', required=True)
   parser_directory.add_argument(
       '-s', '--source', help='Description of the source of the evidence',
       required=False)
@@ -280,21 +293,21 @@ def main():
       '-n', '--name', help='Descriptive name of the evidence', required=False)
 
   # Parser options for CompressedDirectory evidence type
-  parser_compressed_directory = subparsers.add_parser(
+  parser_directory = subparsers.add_parser(
       'compresseddirectory', help='Process a compressed tar file as Evidence')
-  parser_compressed_directory.add_argument(
+  parser_directory.add_argument(
       '-l', '--source_path', help='Local path to the evidence', required=True)
-  parser_compressed_directory.add_argument(
+  parser_directory.add_argument(
       '-s', '--source', help='Description of the source of the evidence',
       required=False)
-  parser_compressed_directory.add_argument(
+  parser_directory.add_argument(
       '-n', '--name', help='Descriptive name of the evidence', required=False)
 
   # Parser options for ChromiumProfile evidence type
   parser_hindsight = subparsers.add_parser(
       'hindsight', help='Process ChromiumProfile as Evidence')
   parser_hindsight.add_argument(
-      '-l', '--local_path', help='Local path to the evidence', required=True)
+      '-l', '--source_path', help='Local path to the evidence', required=True)
   parser_hindsight.add_argument(
       '-f', '--format', help='Output format (supported types are '
       'xlsx, sqlite, jsonl)', default='sqlite')
@@ -308,7 +321,7 @@ def main():
   subparsers.add_parser(
       'listjobs',
       help='List all available Jobs. These Job names can be used by '
-      '--jobs_whitelist and --jobs_blacklist')
+      '--jobs_allowlist and --jobs_denylist')
 
   # PSQ Worker
   parser_psqworker = subparsers.add_parser('psqworker', help='Run PSQ worker')
@@ -332,6 +345,10 @@ def main():
   parser_status.add_argument(
       '-d', '--days_history', default=0, type=int,
       help='Number of days of history to show', required=False)
+  parser_status.add_argument(
+      '-D', '--dump_json', action='store_true',
+      help='Dump JSON status output instead text. Compatible with -d, -u, '
+      '-r and -t flags, but not others')
   parser_status.add_argument(
       '-f', '--force', help='Gatekeeper for --close_tasks', action='store_true',
       required=False)
@@ -358,6 +375,16 @@ def main():
       '-t', '--task_id', help='Show task for given Task ID', required=False)
   parser_status.add_argument(
       '-u', '--user', help='Show task for given user', required=False)
+  parser_status.add_argument(
+      '-i', '--requests', required=False, action='store_true',
+      help='Show all requests from a specified timeframe. The default '
+      'timeframe is 7 days. Please use the -d flag to extend this.')
+  parser_status.add_argument(
+      '-w', '--workers', required=False, action='store_true',
+      help='Show Worker status information from a specified timeframe. The '
+      'default timeframe is 7 days. Please use the -d flag to extend this. '
+      'Additionaly, you can use the -a or --all_fields flag to retrieve the '
+      'full output containing finished and unassigned worker tasks.')
 
   # Server
   subparsers.add_parser('server', help='Run Turbinia Server')
@@ -366,10 +393,17 @@ def main():
 
   # Load the config before final logger setup so we can the find the path to the
   # log file.
-  if args.config_file:
-    config.LoadConfig(config_file=args.config_file)
-  else:
-    config.LoadConfig()
+  try:
+    if args.config_file:
+      config.LoadConfig(config_file=args.config_file)
+    else:
+      config.LoadConfig()
+  except TurbiniaException as exception:
+    print(
+        'Could not load config file ({0!s}).\n{1:s}'.format(
+            exception, config.CONFIG_MSG))
+    sys.exit(1)
+
   if args.log_file:
     config.LOG_FILE = args.log_file
   if args.output_dir:
@@ -385,13 +419,25 @@ def main():
   else:
     log.setLevel(logging.INFO)
 
+  # Enable tasks debugging for supported tasks
+  if args.debug_tasks:
+    config.DEBUG_TASKS = True
+
+  # Enable GCP Stackdriver Logging
+  if config.STACKDRIVER_LOGGING and args.command in ('server', 'psqworker'):
+    google_cloud.setup_stackdriver_handler(
+        config.TURBINIA_PROJECT, args.command)
+
+  config.TURBINIA_COMMAND = args.command
+
   log.info('Turbinia version: {0:s}'.format(__version__))
 
   # Do late import of other needed Turbinia modules.  This is needed because the
   # config is loaded by these modules at load time, and we want to wait to load
   # the config until after we parse the args so that we can use those arguments
   # to point to config paths.
-  from turbinia.client import TurbiniaClient
+  from turbinia import notify
+  from turbinia import client as TurbiniaClientProvider
   from turbinia.client import TurbiniaCeleryClient
   from turbinia.client import TurbiniaServer
   from turbinia.client import TurbiniaCeleryWorker
@@ -399,6 +445,10 @@ def main():
   from turbinia import evidence
   from turbinia.message import TurbiniaRequest
 
+  if config.TASK_MANAGER == 'PSQ':
+    from turbinia.lib import google_cloud
+    from libcloudforensics.providers.gcp import forensics as gcp_forensics
+
   # Print out config if requested
   if args.command == 'config':
     if args.file_only:
@@ -414,10 +464,18 @@ def main():
           "Failed to read config file {0:s}: {1!s}".format(
               config.configSource, exception))
       sys.exit(1)
+  #sends test notification
+  if args.command == 'testnotify':
+    notify.sendmail(
+        config.EMAIL_ADDRESS, 'Turbinia test notification',
+        'This is a test notification')
+    sys.exit(0)
 
-  if args.jobs_whitelist and args.jobs_blacklist:
+  args.jobs_allowlist = [j.lower() for j in args.jobs_allowlist]
+  args.jobs_denylist = [j.lower() for j in args.jobs_denylist]
+  if args.jobs_allowlist and args.jobs_denylist:
     log.error(
-        'A Job filter whitelist and blacklist cannot be specified at the same '
+        'A Job filter allowlist and denylist cannot be specified at the same '
         'time')
     sys.exit(1)
 
@@ -434,16 +492,23 @@ def main():
       log.warning(
           'Cannot open file {0:s} [{1!s}]'.format(args.filter_patterns_file, e))
 
+  # Read yara rules
+  yara_rules = None
+  if (args.yara_rules_file and not os.path.exists(args.yara_rules_file)):
+    log.error('Filter patterns file {0:s} does not exist.')
+    sys.exit(1)
+  elif args.yara_rules_file:
+    try:
+      yara_rules = open(args.yara_rules_file).read()
+    except IOError as e:
+      log.warning(
+          'Cannot open file {0:s} [{1!s}]'.format(args.yara_rules_file, e))
+      sys.exit(1)
+
   # Create Client object
+  client = None
   if args.command not in ('psqworker', 'server'):
-    if config.TASK_MANAGER.lower() == 'celery':
-      client = TurbiniaCeleryClient()
-    elif args.run_local:
-      client = TurbiniaClient(run_local=True)
-    else:
-      client = TurbiniaClient()
-  else:
-    client = None
+    client = TurbiniaClientProvider.get_turbinia_client(args.run_local)
 
   # Make sure run_local flags aren't conflicting with other server/client flags
   server_flags_set = args.server or args.command == 'server'
@@ -473,9 +538,9 @@ def main():
 
     if ((args.project and args.project != config.TURBINIA_PROJECT) or
         (args.zone and args.zone != config.TURBINIA_ZONE)):
-      new_disk = libcloudforensics.create_disk_copy(
+      new_disk = gcp_forensics.CreateDiskCopy(
           args.project, config.TURBINIA_PROJECT, None, config.TURBINIA_ZONE,
-          args.disk_name)
+          disk_name=args.disk_name)
       args.disk_name = new_disk.name
       if args.copy_only:
         log.info('--copy_only specified, so not processing with Turbinia')
@@ -487,34 +552,45 @@ def main():
   # Start Evidence configuration
   evidence_ = None
   if args.command == 'rawdisk':
-    args.name = args.name if args.name else args.local_path
-    local_path = os.path.abspath(args.local_path)
+    args.name = args.name if args.name else args.source_path
+    source_path = os.path.abspath(args.source_path)
     evidence_ = evidence.RawDisk(
-        name=args.name, local_path=local_path,
+        name=args.name, source_path=source_path,
         mount_partition=args.mount_partition, source=args.source)
   elif args.command == 'apfs':
     if not args.password and not args.recovery_key:
       log.error('Neither recovery key nor password is specified.')
       sys.exit(1)
-    args.name = args.name if args.name else args.local_path
-    local_path = os.path.abspath(args.local_path)
+    args.name = args.name if args.name else args.source_path
+    source_path = os.path.abspath(args.source_path)
     evidence_ = evidence.APFSEncryptedDisk(
-        name=args.name, local_path=local_path, recovery_key=args.recovery_key,
+        name=args.name, source_path=source_path, recovery_key=args.recovery_key,
         password=args.password, source=args.source)
   elif args.command == 'bitlocker':
     if not args.password and not args.recovery_key:
       log.error('Neither recovery key nor password is specified.')
       sys.exit(1)
-    args.name = args.name if args.name else args.local_path
-    local_path = os.path.abspath(args.local_path)
+    args.name = args.name if args.name else args.source_path
+    source_path = os.path.abspath(args.source_path)
     evidence_ = evidence.BitlockerDisk(
-        name=args.name, local_path=local_path, recovery_key=args.recovery_key,
+        name=args.name, source_path=source_path, recovery_key=args.recovery_key,
         password=args.password, source=args.source)
   elif args.command == 'directory':
-    args.name = args.name if args.name else args.local_path
-    local_path = os.path.abspath(args.local_path)
-    evidence_ = evidence.Directory(
-        name=args.name, local_path=local_path, source=args.source)
+    args.name = args.name if args.name else args.source_path
+    source_path = os.path.abspath(args.source_path)
+
+    if not config.SHARED_FILESYSTEM:
+      log.info(
+          'A Cloud Only Architecture has been detected. '
+          'Compressing the directory for GCS upload.')
+      source_path = archive.CompressDirectory(
+          source_path, output_path=config.TMP_DIR)
+      args.name = args.name if args.name else source_path
+      evidence_ = evidence.CompressedDirectory(
+          name=args.name, source_path=source_path, source=args.source)
+    else:
+      evidence_ = evidence.Directory(
+          name=args.name, source_path=source_path, source=args.source)
   elif args.command == 'compresseddirectory':
     archive.ValidateTarFile(args.source_path)
     args.name = args.name if args.name else args.source_path
@@ -529,10 +605,16 @@ def main():
         source=args.source)
   elif args.command == 'googleclouddiskembedded':
     args.name = args.name if args.name else args.disk_name
+    parent_evidence_ = evidence.GoogleCloudDisk(
+        name=args.name, disk_name=args.disk_name, project=args.project,
+        mount_partition=args.mount_partition, zone=args.zone,
+        source=args.source)
     evidence_ = evidence.GoogleCloudDiskRawEmbedded(
-        name=args.name, disk_name=args.disk_name,
-        embedded_path=args.embedded_path, mount_partition=args.mount_partition,
-        project=args.project, zone=args.zone, source=args.source)
+        name=args.name, disk_name=args.disk_name, project=args.project,
+        mount_partition=args.mount_partition, zone=args.zone,
+        embedded_path=args.embedded_path,
+        embedded_partition=args.embedded_mount_partition)
+    evidence_.parent_evidence = parent_evidence_
   elif args.command == 'hindsight':
     if args.format not in ['xlsx', 'sqlite', 'jsonl']:
       log.error('Invalid output format.')
@@ -540,30 +622,32 @@ def main():
     if args.browser_type not in ['Chrome', 'Brave']:
       log.error('Browser type not supported.')
       sys.exit(1)
-    args.name = args.name if args.name else args.local_path
-    local_path = os.path.abspath(args.local_path)
+    args.name = args.name if args.name else args.source_path
+    source_path = os.path.abspath(args.source_path)
     evidence_ = evidence.ChromiumProfile(
-        name=args.name, local_path=local_path, output_format=args.format,
+        name=args.name, source_path=source_path, output_format=args.format,
         browser_type=args.browser_type)
   elif args.command == 'rawmemory':
-    args.name = args.name if args.name else args.local_path
-    local_path = os.path.abspath(args.local_path)
+    args.name = args.name if args.name else args.source_path
+    source_path = os.path.abspath(args.source_path)
     evidence_ = evidence.RawMemory(
-        name=args.name, local_path=local_path, profile=args.profile,
+        name=args.name, source_path=source_path, profile=args.profile,
         module_list=args.module_list)
   elif args.command == 'psqworker':
     # Set up root logger level which is normally set by the psqworker command
     # which we are bypassing.
     logger.setup()
-    worker = TurbiniaPsqWorker()
+    worker = TurbiniaPsqWorker(
+        jobs_denylist=args.jobs_denylist, jobs_allowlist=args.jobs_allowlist)
     worker.start()
   elif args.command == 'celeryworker':
     logger.setup()
-    worker = TurbiniaCeleryWorker()
+    worker = TurbiniaCeleryWorker(
+        jobs_denylist=args.jobs_denylist, jobs_allowlist=args.jobs_allowlist)
     worker.start()
   elif args.command == 'server':
     server = TurbiniaServer(
-        jobs_blacklist=args.jobs_blacklist, jobs_whitelist=args.jobs_whitelist)
+        jobs_denylist=args.jobs_denylist, jobs_allowlist=args.jobs_allowlist)
     server.start()
   elif args.command == 'status':
     region = config.TURBINIA_REGION
@@ -581,6 +665,12 @@ def main():
         )
         sys.exit(1)
 
+    if args.dump_json and (args.statistics or args.requests or args.workers):
+      log.info(
+          'The --dump_json flag is not compatible with --statistics, '
+          '--reqeusts, or --workers flags')
+      sys.exit(1)
+
     if args.statistics:
       print(
           client.format_task_statistics(
@@ -599,13 +689,35 @@ def main():
           '--wait requires --request_id, which is not specified. '
           'turbiniactl will exit without waiting.')
 
+    if args.requests:
+      print(
+          client.format_request_status(
+              instance=config.INSTANCE_ID, project=config.TURBINIA_PROJECT,
+              region=region, days=args.days_history,
+              all_fields=args.all_fields))
+      sys.exit(0)
+
+    if args.workers:
+      print(
+          client.format_worker_status(
+              instance=config.INSTANCE_ID, project=config.TURBINIA_PROJECT,
+              region=region, days=args.days_history,
+              all_fields=args.all_fields))
+      sys.exit(0)
+
+    if args.dump_json:
+      output_json = True
+    else:
+      output_json = False
     print(
         client.format_task_status(
             instance=config.INSTANCE_ID, project=config.TURBINIA_PROJECT,
             region=region, days=args.days_history, task_id=args.task_id,
             request_id=args.request_id, user=args.user,
             all_fields=args.all_fields, full_report=args.full_report,
-            priority_filter=args.priority_filter))
+            priority_filter=args.priority_filter, output_json=output_json))
+    sys.exit(0)
+
   elif args.command == 'listjobs':
     log.info('Available Jobs:')
     client.list_jobs()
@@ -651,10 +763,14 @@ def main():
     request.evidence.append(evidence_)
     if filter_patterns:
       request.recipe['filter_patterns'] = filter_patterns
-    if args.jobs_blacklist:
-      request.recipe['jobs_blacklist'] = args.jobs_blacklist
-    if args.jobs_whitelist:
-      request.recipe['jobs_whitelist'] = args.jobs_whitelist
+    if args.jobs_denylist:
+      request.recipe['jobs_denylist'] = args.jobs_denylist
+    if args.jobs_allowlist:
+      request.recipe['jobs_allowlist'] = args.jobs_allowlist
+    if yara_rules:
+      request.recipe['yara_rules'] = yara_rules
+    if args.debug_tasks:
+      request.recipe['debug_tasks'] = args.debug_tasks
     if args.recipe_config:
       for pair in args.recipe_config:
         try:
